/* TSL2591 Digital Light Sensor */
/* Dynamic Range: 600M:1 */
/* Maximum Lux: 88K */
/* Addition of LCD display to the basic read routine*/

#include <Wire.h>
#include <Adafruit_Sensor.h>
#include "Adafruit_TSL2591.h"
#include <hd44780.h>                       // main hd44780 header
#include <hd44780ioClass/hd44780_I2Cexp.h> // i2c expander i/o class header

// connect SCL to I2C Clock A5
// connect SDA to I2C Data A4
// connect momentary switches to pins D2 and D3 for changing of sensitivity
// connect Vin to 3.3-5V DC
// connect GROUND to common ground


const int LCD_COLS = 16;
const int LCD_ROWS = 2;
const byte interruptPin1 = 2;
const byte interruptPin2 = 3;
volatile byte state = LOW;
volatile byte sens = 1; // sens has values 1 to 4 for Low Med High Xtra gain
int delay_time = 500;
unsigned long t1;
unsigned long t2;


Adafruit_TSL2591 tsl = Adafruit_TSL2591(2591); // pass in a number for the sensor identifier (for your use later)


hd44780_I2Cexp lcd; // automatic detection i2c address for the lcd
int status;


// set the lcd rows and columns and check the procedure worked OK
void lcd_setup()
{
  status = lcd.begin(LCD_COLS, LCD_ROWS);
  if (status) // non zero status means it was unsuccesful
  {
    status = -status; // convert negative status value to positive number

    // begin() failed so blink error code using the onboard LED if possible
    hd44780::fatalError(status); // does not return
  } 0
  ;
}

// initalization was successful, the backlight should be on now



/**************************************************************************/
/*
    Displays some basic information on this sensor from the unified
    sensor API sensor_t type (see Adafruit_Sensor for more information)
*/
/**************************************************************************/
void displaySensorDetails(void)
{
  sensor_t sensor;
  tsl.getSensor(&sensor);
  Serial.println(F("------------------------------------"));
  Serial.print  (F("Sensor:       ")); Serial.println(sensor.name);
  Serial.print  (F("Driver Ver:   ")); Serial.println(sensor.version);
  Serial.print  (F("Unique ID:    ")); Serial.println(sensor.sensor_id);
  Serial.print  (F("Max Value:    ")); Serial.print(sensor.max_value); Serial.println(F(" lux"));
  Serial.print  (F("Min Value:    ")); Serial.print(sensor.min_value); Serial.println(F(" lux"));
  Serial.print  (F("Resolution:   ")); Serial.print(sensor.resolution, 4); Serial.println(F(" lux"));
  Serial.println(F("------------------------------------"));
  Serial.println(F(""));
}



/**************************************************************************/
/*
    Configures the gain and integration time for the TSL2591
*/
/**************************************************************************/
void configureSensor(void)
{
  // You can change the gain on the fly, to adapt to brighter/dimmer light situations
  tsl.setGain(TSL2591_GAIN_LOW);    // 1x gain (bright light)
  sens = 1;
  //tsl.setGain(TSL2591_GAIN_MED);      // 25x gain
  //tsl.setGain(TSL2591_GAIN_HIGH);   // 428x gain
  //tsl.setGain(TSL2591_GAIN_MAX);   // 9876x gain

  // Changing the integration time gives you a longer time over which to sense light
  // longer timelines are slower, but are good in very low light situtations!
  tsl.setTiming(TSL2591_INTEGRATIONTIME_300MS); // Alternatives are 100, 200, 300, 400, 500, 600

  /* Display the gain and integration time for reference sake */
  Serial.println(F("------------------------------------"));
  Serial.print  (F("Gain:         "));
  tsl2591Gain_t gain = tsl.getGain();
  switch (gain)
  {
    case TSL2591_GAIN_LOW:
      Serial.println(F("1x (Low)"));
      break;
    case TSL2591_GAIN_MED:
      Serial.println(F("25x (Medium)"));
      break;
    case TSL2591_GAIN_HIGH:
      Serial.println(F("428x (High)"));
      break;
    case TSL2591_GAIN_MAX:
      Serial.println(F("9876x (Max)"));
      break;
  }
  Serial.print  (F("Timing:       "));
  Serial.print((tsl.getTiming() + 1) * 100, DEC);
  Serial.println(F(" ms"));
  Serial.println(F("------------------------------------"));
  Serial.println(F(""));
}


/**************************************************************************/
/*
    Program entry point for the Arduino sketch
*/
/**************************************************************************/
void setup(void)
{
  Serial.begin(9600);
  displaySensorDetails();
  configureSensor();
  status = lcd.begin(LCD_COLS, LCD_ROWS);
  lcd.clear();
  lcd.leftToRight();
  lcd.setCursor(0, 0);
  lcd.print("Visible ");
  lcd.setCursor(0, 2);
  lcd.print("Lux ");
  lcd.setCursor(13, 0);
  lcd.print("S=");
  lcd.print(sens);

  pinMode(interruptPin1, INPUT_PULLUP);
  pinMode(interruptPin2, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(interruptPin1), interrupt1, FALLING);
  attachInterrupt(digitalPinToInterrupt(interruptPin2), interrupt2, FALLING);

  Serial.println(F("Starting Adafruit TSL2591 Test!"));

  if (tsl.begin())
  {
    Serial.println(F("Found a TSL2591 sensor"));
  }
  else
  {
    Serial.println(F("No sensor found ... check your wiring?"));
    while (1);
  }

  // Now we're ready to get readings ... move on to loop()!
}


/**************************************************************************/
/*
    Show how to read IR and Full Spectrum at once and convert to lux
*/
/**************************************************************************/
void advancedRead(void)
{
  // More advanced data read example. Read 32 bits with top 16 bits IR, bottom 16 bits full spectrum
  // That way you can do whatever math and comparisons you want!

  //tsl.setGain(TSL2591_GAIN_LOW);    // 1x gain (bright light)
  //tsl.setGain(TSL2591_GAIN_MED);      // 25x gain
  //tsl.setGain(TSL2591_GAIN_HIGH);   // 428x gain
  //tsl.setGain(TSL2591_GAIN_MAX);   // 9876x gain
  float lux;
  uint32_t lum = tsl.getFullLuminosity();
  uint16_t ir, full;
  ir = lum >> 16;
  full = lum & 0xFFFF;
  lux = tsl.calculateLux(full, ir);


 // Serial.print(F("[ ")); Serial.print(millis()); Serial.print(F(" ms ] "));
  Serial.print(F("  IR: ")); Serial.print(ir);  Serial.print(F("  "));
  Serial.print(F("Full: ")); Serial.print(full); Serial.print(F("  "));
  Serial.print(F("Visible: ")); Serial.print(full - ir); Serial.print(F("  "));
  Serial.print(F("Lux: ")); Serial.print((lux), 6);

  lcd.setCursor(9, 0);
  lcd.print(full - ir);
  lcd.setCursor(5, 1);
  lcd.print(tsl.calculateLux(full, ir), 6);
}


/**************************************************************************/
/*
    Arduino loop function, called once 'setup' is complete (your own code
    should go here)
*/
/**************************************************************************/
void loop(void)
{
    t1 = millis();
    if (sens >= 5) sens = 4;
    if (sens <= 0) sens = 1;
    if (sens == 1) tsl.setGain(TSL2591_GAIN_LOW);    // 1x gain high light levels
    if (sens == 2) tsl.setGain(TSL2591_GAIN_MED);    // 25x gain
    if (sens == 3) tsl.setGain(TSL2591_GAIN_HIGH);   // 428x gain
    if (sens == 4) tsl.setGain(TSL2591_GAIN_MAX);    // 9876x gain  dark conditions
    
    Serial.print  (F("Gain: "));
    tsl2591Gain_t gain = tsl.getGain();
    Serial.print(gain);
    lcd.setCursor(15, 0);
    lcd.print(sens);
  advancedRead();
//  Serial.println(F("In the loop after read"));
  delay(1000);
  Serial.print(F("  interval= "));
  int interval = (millis()-t1);
  Serial.println(interval);
}


void interrupt1() 
{
  Serial.println(F("interupt one"));
  sens ++;
  state = !state;
}

void interrupt2()
{
  Serial.println(F("interrupt two"));
  sens --;
  state = !state;
}
